#!/usr/bin/env python
# encoding: utf-8
"""
@author: simin. Gong
@license: (C) Copyright 2013-2023 Spacemit Corporation Limited.
@contact: simin.gong@spacemit.com
@file: LLMNamespace.py
@date: 2024/11/28 09:25 
@desc:
"""

from flask_socketio.namespace import Namespace
from ollama import chat
import threading

class OllamaNamespace(Namespace):

    def __init__(self, namespace=None):
        super(OllamaNamespace, self).__init__(namespace)
        self.chat_lock = threading.Lock()
        self.model_name = self.load_model_name()
        
    def load_model_name(self):
        try:
            with open('/etc/llm-model', 'r') as file:
                for line in file:
                    if line.startswith("ModelName="):
                        model_name = line.split("=", 1)[1].strip()  # 提取等号后面的内容
                        if model_name:
                            return model_name
            return 'qwen'  # 如果没有找到则返回默认模型名称
        except Exception as e:
            print(f"读取模型名称时出错: {e}")
            return 'qwen'  # 读取失败时使用默认模型名称

    def on_connect(self):
        pass

    def on_disconnect(self):
        pass

    def on_chat(self, data):
        self.chat_lock.acquire()
        try:
            params = data.split('&') 
            for param in params:
                key, value = param.split('=')
                if key == 'model':
                    self.model_name = value
                elif key == 'msg':
                    msg = value
            response = chat(model=self.model_name, stream=True, messages=[
                {
                    'role': 'user',
                    'content': msg,
                },
            ])
            res = ''
            for part in response:
                res += part['message']['content']
                self.emit('chat-response', res)
        except Exception as e:
            print(e)
            self.emit('chat-error', f'{e}')
        finally:
            self.emit('chat-over')
        self.chat_lock.release()